/*-- sun.c
     Solve long file
     This version: 
        - First solution only
        - Counts the total number of visits
        - Counts CPU time
*/
#include <stdio.h>
#include <time.h>
#define MINC 5000000
#define NONE (-1)

clock_t start, end;
double cpu_time;

int m[9][9],
  nodes=0,         // Total number of tree nodes
  cpu=0,           // First time: do not display cpu time
  cnt[9][9]={{0}}, // Number of visits to this node
  br[9][9]={{0}};  // Number of sons by node

//--------------- read puzzle --------------------
init(void){
  int i,j;
  nodes=0;
}
    
//--------------- read puzzle --------------------
int nextd(void){
  int c;
  do{
    c=getchar();
    if(c==EOF)
      return NONE;
    if(c=='-')
      return 0;
  }
  while(c<'0' || c>'9');
  return c-'0';
}

int readm(void){
  int i,j,c;
  for(i=0;i<9;i++)
    for(j=0;j<9;j++){
      c=nextd();
      if(c==NONE)
	return (0);
      m[i][j]=c;
    }
  return (1);
}

//--------------- print matrix ------------------
//--------- used only for debug -----------------
void display(void){
  int i,j;
  end = clock();
  cpu_time = ((double) (end - start)) / CLOCKS_PER_SEC;
  printf("\n");
  for(i=0;i<9;i++){
    for(j=0;j<9;j++){
      if(m[i][j]>0)
	printf(" %d",m[i][j]);
      else
	printf(" -");
    if(j%3==2)
      printf(" ");
    }      
    printf("\n");
    if(i%3==2)
      printf("\n");
  }
  printf("\n");
  //  if(!cpu){
  //    cpu=1;
  //    return;
  //  }
  //  printf("      CPU time:       %10.3f seconds\n",cpu_time);
  //  printf("      No. of visits:      %10d\n",nodes);
  //  printf("\n");
}

//--------------- print nodes with average branching >= MINC ----
//--------- used only for debug -----------------
void freqs(void){
  int i,j,sum=0;
  float branch;
  printf("\n");
  for(i=0;i<9;i++)
    for(j=0;j<9;j++)
      sum+=cnt[i][j];
  printf("Visits = %9d\n",sum);
  for(i=0;i<9;i++){
    for(j=0;j<9;j++){
	if(cnt[i][j]>=MINC)
	printf("x=%1d y=%1d (%1d:%1d) Cnt=%5d\n",
	     j+1,10-(i+1),
	     i+1,j+1,
             cnt[i][j]);
    }   
  }
  printf("\n");
}

//------------- test constraints of cell m[a][b] ----
int ok(int a,int b){
  int v=m[a][b],i,j,iq,jq;
  // printf("(%d) ",v);
  for(i=0;i<9;i++)       //-- test line
    if( i!=a && m[i][b]==v)
	return 0;
  for(j=0;j<9;j++)       //-- test column
    if( j!=b && m[a][j]==v)
	return 0;
  iq=a/3; jq=b/3;        //-- test square
  for(i=0;i<3;i++) 
    for(j=0;j<3;j++) 
      if( (3*iq+i!=a || 3*jq+j!=b) && m[3*iq+i][3*jq+j]==v)
	return 0;
  return 1;
}


//-----------------------------------------------------
//-- main DFS recursive function
//   only first solution
//-----------------------------------------------------
int solve(int i,int j){
  int v;
  if(j==9){   //-- is next cell in next row?
    j=0;
    i++;
  }

  if(i==9 && j==0){ //-- a solution was found
    display();
    return (1);
  }

  nodes++;
  //  cnt[i][j]++;      //-- one visit to this node

  if(m[i][j]>0){    //-- clue there?
    // br[i][j]++;     
    return (solve(i,j+1));
  }

  // Empty. Try all possible values
  for(v=1;v<=9;v++){
    m[i][j]=v;
    if(ok(i,j)){
      // printf("* ");
      // br[i][j]++;    // One more branch
      if (solve(i,j+1))
	return (1);
    }
    m[i][j]=0;
  }
  return (0);
}

//--------------------------------------------------
main(){
  while(readm()){
    init();
    printf("--------------------------------------\n");
    display();
    start = clock();
    solve(0,0);
    end = clock();
    cpu_time = ((double) (end - start)) / CLOCKS_PER_SEC;
    printf("      Total CPU time: %10.3f seconds\n",cpu_time);
    printf("      Total no. of visits: %9d\n",nodes);
    //  freqs();
  }
}
