/*-- sun.c
     Solve sudoku puzzles
     This version: 
        - Counts number of branching by node
        - Counts number of search nodes
        - Counts CPU time
	- Selects nodes with no. of visits average branching >= 5000000
*/
#include <stdio.h>
#include <time.h>
#define MINC 5000000

clock_t start, end;
double cpu_time;

int m[9][9],
  nodes=0,         // Total number of tree nodes
  cpu=0,           // First time: do not display cpu time
  cnt[9][9]={{0}}, // Number of visits to this node
  br[9][9]={{0}};  // Number of sons by node

//--------------- read puzzle --------------------
int nextd(void){
  int c;
  do
    c=getchar();
  while(c!='-' & (c<'0' || c>'9'));
  if(c=='-')
    c='0';
  return c-'0';
}


void readm(void){
  int i,j;
  for(i=0;i<9;i++)
    for(j=0;j<9;j++)
      m[i][j]=nextd();
}

//--------------- print matrix ------------------
void display(void){
  int i,j;
  end = clock();
  cpu_time = ((double) (end - start)) / CLOCKS_PER_SEC;
  printf("\n");
  for(i=0;i<9;i++){
    for(j=0;j<9;j++){
      if(m[i][j]>0)
	printf(" %d",m[i][j]);
      else
	printf(" -");
    if(j%3==2)
      printf(" ");
    }      
    printf("\n");
    if(i%3==2)
      printf("\n");
  }
  printf("\n");
  if(!cpu){
    cpu=1;
    return;
  }
  printf("      CPU time:       %10.3f seconds\n",cpu_time);
  printf("      No. of visits:      %10d\n",nodes);
  printf("\n");
}

//--------------- print nodes with average branching >= MINC ----
void freqs(void){
  int i,j,sum=0;
  float branch;
  printf("\n");
  for(i=0;i<9;i++)
    for(j=0;j<9;j++)
      sum+=cnt[i][j];
  printf("Visits = %9d\n",sum);
  for(i=0;i<9;i++){
    for(j=0;j<9;j++){
	if(cnt[i][j]>=MINC)
	printf("x=%1d y=%1d (%1d:%1d) Cnt=%5d\n",
	     j+1,10-(i+1),
	     i+1,j+1,
             cnt[i][j]);
    }   
  }
  printf("\n");
}

//------------- test constraints of cell m[a][b] ----
int ok(int a,int b){
  int v=m[a][b],i,j,iq,jq;
  for(i=0;i<9;i++)       //-- test line
    if( i!=a && m[i][b]==v)
	return 0;
  for(j=0;j<9;j++)       //-- test column
    if( j!=b && m[a][j]==v)
	return 0;
  iq=a/3; jq=b/3;        //-- test square
  for(i=0;i<3;i++) 
    for(j=0;j<3;j++) 
      if( (3*iq+i!=a || 3*jq+j!=b) && m[3*iq+i][3*jq+j]==v)
	return 0;
  return 1;
}


//-----------------------------------------------------
//-- main DFS recursive function
//-----------------------------------------------------
void solve(int i,int j){
  int v;
  if(j==9){   //-- is next cell in next row?
    j=0;
    i++;
  }

  if(i==9 && j==0){ //-- a solution was found
    display();
    return;
  }

  nodes++;
  cnt[i][j]++;      //-- one visit to this node

  if(m[i][j]>0){    //-- clue there?
    br[i][j]++;     
    solve(i,j+1);
    return;
  }

  // Empty. Try all possible values
  for(v=1;v<=9;v++){
    m[i][j]=v;
    if(ok(i,j)){
      br[i][j]++;    // One more branch
      solve(i,j+1);
    }
    m[i][j]=0;
  }
  return;
}

//--------------------------------------------------
main(){
  readm();
  display();
  start = clock();
  solve(0,0);
  end = clock();
  cpu_time = ((double) (end - start)) / CLOCKS_PER_SEC;
  printf("      Total CPU time: %10.3f seconds\n",cpu_time);
  printf("      Total no. of visits: %9d\n",nodes);
  freqs();
}
